using System.Text;
using Amazon.Textract;
using Amazon.Textract.Model;

public static class AnalyzeDocumentResponseExtension
{
    public static Dictionary<string, string> GetKeyValuePairs(this AnalyzeDocumentResponse analyzeDocumentResponse)
    {
        var key_value_set = analyzeDocumentResponse.Blocks.Where(b => b.BlockType == BlockType.KEY_VALUE_SET && b.Relationships.Count == 2).ToList();
        var keyValuePairs = new Dictionary<string, string>();
        foreach (var block in key_value_set)
        {
            var keyText = GetKeyText(block, analyzeDocumentResponse.Blocks);
            var valueText = GetValueText(block, analyzeDocumentResponse.Blocks);
            bool resultOfAdd = keyValuePairs.TryAdd($"{keyText}", valueText);
            if (!resultOfAdd)
            {
                Console.WriteLine($"Key already exists: {block.Page}:{keyText}");
            }
        }

        return keyValuePairs;

        string GetKeyText(Block keyValueSet, List<Block> allBlocks)
        {
            StringBuilder keyText = new StringBuilder();

            foreach (var keyPortionId in keyValueSet.Relationships[1].Ids)
            {
                var key = allBlocks.Where(b => b.Id == keyPortionId).FirstOrDefault();
                keyText.Append($"{key.Text} ");
            }
            return keyText.ToString();
        }

        string GetValueText(Block keyValueSet, List<Block> allBlocks)
        {
            var valueLink = allBlocks.Where(b => b.Id == keyValueSet.Relationships[0].Ids[0]).FirstOrDefault();

            StringBuilder valueText = new StringBuilder();
            if (valueLink.Relationships.Any())
            {
                foreach (var valuePortionId in valueLink.Relationships[0].Ids)
                {
                    var value = allBlocks.Where(b => b.Id == valuePortionId).FirstOrDefault();
                    valueText.Append($"{value.Text} ");
                }
                return valueText.ToString();
            }
            return String.Empty;
        }
    }
}